import { useState, useEffect } from "react";
import { LabeledIcon } from "@ess-ics/ce-ui-common";
import Popover from "../../common/Popover";
import { object, bool, array, func } from "prop-types";
import { useTheme, Skeleton } from "@mui/material";
import { PlayCircleFilled, PauseCircleFilled } from "@mui/icons-material";
import { StatusPopoverContent } from "./StatusPopoverContent";
import { StatusBadge } from "./StatusBadge";
import { StatusIcon } from "./StatusIcon";
import { statusesWithAlerts, statusConfig } from "./StatusData";

const propsTypes = {
  state: object,
  alert: array,
  hideAlerts: bool,
  getStatusFcn: func
};

export function Status({ id, state, alert, hideAlerts = false, getStatusFcn }) {
  const [status, setStatus] = useState(null);
  const theme = useTheme();

  useEffect(() => {
    if (state) {
      setStatus(getStatusFcn(state, alert));
    }
  }, [state, alert, getStatusFcn]);

  return (
    <>
      {status ? (
        <Popover
          id={`ioc-status-popover-${id}`}
          renderOwningComponent={({
            ariaOwns,
            ariaHasPopup,
            handlePopoverOpen,
            handlePopoverClose
          }) => (
            <div
              aria-owns={ariaOwns}
              aria-haspopup={ariaHasPopup}
              onMouseEnter={handlePopoverOpen}
              onMouseLeave={handlePopoverClose}
              style={{ color: theme.palette.status.icons }}
            >
              {!hideAlerts && statusesWithAlerts.includes(status) ? (
                <StatusBadge
                  status={status}
                  theme={theme}
                >
                  <StatusIcon
                    alerts={alert?.alerts ?? []}
                    status={status}
                  />
                </StatusBadge>
              ) : (
                <StatusIcon
                  alerts={alert?.alerts ?? []}
                  status={status}
                />
              )}
            </div>
          )}
          popoverContents={
            <StatusPopoverContent
              title={statusConfig[status].title}
              alerts={hideAlerts ? [] : (alert?.alerts ?? [])}
            />
          }
          anchorOrigin={{
            vertical: "top",
            horizontal: "right"
          }}
        />
      ) : (
        <Skeleton
          variant="circular"
          height={24}
          width={24}
        />
      )}
    </>
  );
}

Status.propsTypes = propsTypes;

export function CommandTypeIcon({
  type,
  colorStyle = "colors",
  labelPosition = "tooltip"
}) {
  const theme = useTheme();

  const commandTypeColors = {
    start: theme.palette.status.ok.main,
    stop: theme.palette.status.fail.main
  };

  const commandTypeBlack = {
    start: theme.palette.status.icons,
    stop: theme.palette.status.icons
  };

  const colorConfig = {
    black: commandTypeBlack,
    colors: commandTypeColors
  };

  const commandTypeIcons = {
    start: {
      title: "Set to active",
      icon: PlayCircleFilled
    },
    stop: {
      title: "Set to inactive",
      icon: PauseCircleFilled
    }
  };

  const iconStyle = { fill: colorConfig[colorStyle][type.toLowerCase()] };
  const iconTitle = commandTypeIcons[type.toLowerCase()].title;
  const statusIcon = commandTypeIcons[type.toLowerCase()].icon;

  return (
    <LabeledIcon
      label={iconTitle}
      LabelProps={{ noWrap: true }}
      labelPosition={labelPosition}
      Icon={statusIcon}
      IconProps={{ style: { iconStyle } }}
    />
  );
}
