import { Grid, IconButton, Stack } from "@mui/material";
import ArrowBackIcon from "@mui/icons-material/ArrowBack";
import { useCallback, useContext, useEffect, useMemo, useState } from "react";
import { IOCLiveStatus } from "../../components/IOC/IOCLiveStatus";
import { IOCManage } from "../../components/IOC/IOCManage";
import { useNavigate } from "react-router-dom";
import IOCAdmin from "../../components/IOC/IOCAdmin";
import {
  applicationTitle,
  initRequestParams
} from "../../components/common/Helper";
import {
  GlobalAppBarContext,
  useAPIMethod,
  useIsCurrentUserPermitted,
  TabPanel,
  usePagination,
  usePolling
} from "@ess-ics/ce-ui-common";
import { apiContext } from "../../api/DeployApi";
import { ROWS_PER_PAGE } from "../../constants";

const IOC_POLL_INTERVAL = 10000;
export function IOCDetailsView({ ioc, getIOC, abortGetIOC, loading }) {
  const { setTitle } = useContext(GlobalAppBarContext);
  useEffect(() => {
    if (ioc) {
      setTitle(applicationTitle(`IOC Details: ${ioc.namingName}`));
    }
  }, [ioc, setTitle]);

  const [buttonDisabled, setButtonDisabled] = useState(false);
  const navigate = useNavigate();

  const client = useContext(apiContext);
  const {
    value: jobs,
    wrapper: getJobs,
    loading: jobsLoading,
    dataReady: jobsDataReady,
    abort: abortGetJobs
  } = useAPIMethod({
    fcn: client.apis.Jobs.listJobs,
    call: false
  });

  const ongoingCommandParams = useMemo(
    () => ({
      ioc_id: ioc.id,
      type: "COMMAND",
      status: "ONGOING"
    }),
    [ioc]
  );

  const {
    value: ongoingCommand,
    wrapper: getOngoingCommand,
    loading: ongoingCommandLoading,
    dataReady: ongoingCommandDataReady,
    abort: abortGetOngoingCommand
  } = useAPIMethod({
    fcn: client.apis.Jobs.listJobs,
    params: ongoingCommandParams,
    call: false
  });

  const { pagination: jobPagination, setPagination: setJobPagination } =
    usePagination({
      rowsPerPageOptions: ROWS_PER_PAGE,
      initLimit: ROWS_PER_PAGE[0],
      initPage: 0
    });
  const [tabIndex, setTabIndex] = useState(0);

  // update pagination whenever search result total pages change
  useEffect(() => {
    setJobPagination({ totalCount: jobs?.totalCount ?? 0 });
  }, [setJobPagination, jobs?.totalCount]);

  // Invoked by Table on change to pagination
  const onPage = useCallback(
    (params) => {
      setJobPagination(params);
      abortGetJobs();
    },
    [abortGetJobs, setJobPagination]
  );

  usePolling(getIOC, loading, IOC_POLL_INTERVAL, abortGetIOC);
  usePolling(
    getOngoingCommand,
    ongoingCommandLoading || !ongoingCommandDataReady,
    IOC_POLL_INTERVAL,
    abortGetOngoingCommand
  );

  useEffect(() => {
    setButtonDisabled(Boolean(ioc?.operationInProgress));
  }, [ioc?.operationInProgress]);

  const callGetJobs = useCallback(() => {
    let requestParams = initRequestParams(jobPagination);
    requestParams.ioc_id = ioc.id;
    getJobs(requestParams);
  }, [getJobs, ioc.id, jobPagination]);

  // Submit new search whenever pagination or ioc changes
  useEffect(() => {
    callGetJobs();

    return () => {
      abortGetJobs();
    };
  }, [callGetJobs, abortGetJobs]);

  usePolling(callGetJobs, jobsLoading, 30000, abortGetJobs);

  const handleClick = () => {
    navigate(-1);
  };

  const setButtonDisabledAndUpdate = useCallback(
    (isDisabled) => {
      setButtonDisabled(isDisabled);
      getIOC();
    },
    [getIOC]
  );

  const tabs = [
    {
      label: "Status",
      content: <IOCLiveStatus ioc={ioc} />
    }
  ];

  const isPermittedManagement = useIsCurrentUserPermitted({
    allowedRoles: ["DeploymentToolAdmin", "DeploymentToolIntegrator"]
  });

  const isPermittedAdmin = useIsCurrentUserPermitted({
    allowedRoles: ["DeploymentToolAdmin"]
  });

  if (isPermittedManagement) {
    tabs.push({
      label: "Management",
      content: (
        <IOCManage
          ioc={ioc}
          getIOC={getIOC}
          buttonDisabled={buttonDisabled}
          currentCommand={
            ongoingCommand?.jobs?.length > 0 ? ongoingCommand.jobs[0] : null
          }
          jobs={jobs?.jobs}
          jobsLoading={jobsLoading || !jobsDataReady}
          getJobs={getJobs}
          setButtonDisabled={setButtonDisabledAndUpdate}
          pagination={jobPagination}
          onPage={onPage}
        />
      )
    });
  }
  if (isPermittedAdmin) {
    tabs.push({
      label: "Admin",
      content: (
        <IOCAdmin
          ioc={ioc}
          getIOC={getIOC}
          resetTab={() => setTabIndex(0)}
          buttonDisabled={buttonDisabled}
          setButtonDisabled={setButtonDisabled}
        />
      )
    });
  }

  return (
    <Grid
      container
      spacing={1}
    >
      <Grid
        item
        xs={12}
        style={{ paddingBottom: 0 }}
      >
        <TabPanel
          tabs={tabs}
          initialIndex={tabIndex}
          onTabChange={(index) => setTabIndex(index)}
          TabsProps={{ centered: true, sx: { flex: 1 } }}
          renderTabs={(tabs) => (
            <Stack
              flexDirection="row"
              justifyContent="space-between"
            >
              <IconButton
                color="inherit"
                onClick={handleClick}
                size="large"
              >
                <ArrowBackIcon />
              </IconButton>
              {tabs}
            </Stack>
          )}
        />
      </Grid>
    </Grid>
  );
}
