import { useState, useEffect, useCallback, useContext } from "react";
import { Container, Grid, Tabs, Tab } from "@mui/material";
import { HostTable } from "../../components/host/HostTable";
import {
  GlobalAppBarContext,
  RootPaper,
  useAPIMethod,
  usePagination,
  SearchBar
} from "@ess-ics/ce-ui-common";
import {
  applicationTitle,
  initRequestParams
} from "../../components/common/Helper";
import { useSearchParams } from "react-router-dom";
import { apiContext } from "../../api/DeployApi";
import { ROWS_PER_PAGE } from "../../constants";

export function HostListView() {
  const { setTitle } = useContext(GlobalAppBarContext);
  useEffect(() => setTitle(applicationTitle("IOC hosts")), [setTitle]);

  const client = useContext(apiContext);
  const {
    value: hosts,
    wrapper: getHosts,
    dataReady,
    loading,
    abort
  } = useAPIMethod({
    fcn: client.apis.Hosts.listHosts,
    call: false
  });

  const [searchParams, setSearchParams] = useSearchParams({ query: "" });
  const [tabIndex, setTabIndex] = useState(0);
  const [hostFilter, setHostFilter] = useState("ALL");

  const handleTabChange = (tab) => {
    if (tab === 0) {
      setHostFilter("ALL");
    } else if (tab === 1) {
      setHostFilter("IOCS_DEPLOYED");
    } else if (tab === 2) {
      setHostFilter("NO_IOCS");
    }
    setTabIndex(tab);
  };

  const { pagination, setPagination } = usePagination({
    rowsPerPageOptions: ROWS_PER_PAGE,
    initLimit: ROWS_PER_PAGE[0],
    initPage: 0
  });

  // update pagination whenever search result total pages change
  useEffect(() => {
    setPagination({ totalCount: hosts?.totalCount ?? 0 });
  }, [setPagination, hosts?.totalCount]);

  // Request new search results whenever search or pagination changes
  useEffect(() => {
    let requestParams = initRequestParams(pagination);
    requestParams.filter = hostFilter;
    requestParams.text = searchParams.get("query");
    getHosts(requestParams);

    return () => {
      abort();
    };
  }, [getHosts, hostFilter, searchParams, pagination, abort]);

  // Callback for searchbar, called whenever user updates search
  const setSearch = useCallback(
    (query) => {
      setSearchParams({ query }, { replace: true });
    },
    [setSearchParams]
  );

  // Invoked by Table on change to pagination
  const onPage = (params) => {
    setPagination(params);
    abort();
  };

  const content = (
    <SearchBar
      search={setSearch}
      query={searchParams.get("query")}
      loading={loading}
    >
      <HostTable
        hosts={hosts?.netBoxHosts ?? []}
        loading={loading || !dataReady}
        pagination={pagination}
        onPage={onPage}
      />
    </SearchBar>
  );

  return (
    <RootPaper>
      <Grid
        container
        spacing={1}
      >
        <Grid
          container
          spacing={1}
          component={Container}
          justifyContent="space-between"
          alignItems="center"
          style={{ display: "flex" }}
        >
          <Grid item>
            <Tabs
              value={tabIndex}
              onChange={(_, tab) => handleTabChange(tab)}
            >
              <Tab label="All" />
              <Tab label="Only hosts with IOCs" />
              <Tab label="Only hosts without IOCs" />
            </Tabs>
          </Grid>
        </Grid>
        <Grid
          item
          xs={12}
          md={12}
        >
          {content}
        </Grid>
      </Grid>
    </RootPaper>
  );
}
